/**
  ******************************************************************************
  * @file    qspi.h
  * @author  CMS Application Team
  * @version Vx.x.x
  * @date    24-April-2022
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Quad Serial Perripheral Interface(QSPI):
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 Cmsemicon.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/

#ifndef QSPI_H
#define QSPI_H
#include "CMS32H3201.h"
#include "stdint.h"
#include "common.h"
/**************************************************************************
* QSPI Registers defination
**************************************************************************/
#define BYTE_FIELD(value, byte) ((value & (0xFF << 8 * byte )) >> 8 * byte)

/**************************************************************************
* Bit Fields defination
**************************************************************************/
// Status Register-1
#define BUSY_Pos  (0x00U)
#define BUSY_Msk  (0x01U)
#define WEL_Pos   (0x01U)
#define WEL_Msk   (0x02U)
#define BP_Pos    (0x04U)
#define BP_Msk    (0x30U)
#define TB_Pos    (0x06U)
#define TB_Msk    (0x40U)
#define BPL_Pos   (0x07U)
#define BPS_Msk   (0x80U)
// Status Register-2
#define QE_Pos    (0x01U)
#define QE_Msk    (0x02U)

/**************************************************************************
* Typedef defination
**************************************************************************/
typedef enum
{
	STATUS_REG1 = 1,
	STATUS_REG2 = 2
} status_t;

typedef enum
{
	QSPI_QSSL_CLK1    = 0x0000,  //QSPI QSSL high level with 1 QSSPCLK
	QSPI_QSSL_CLK2    = 0x0001,  //QSPI QSSL high level with 2 QSSPCLK
	QSPI_QSSL_CLK3    = 0x0002,  //QSPI QSSL high level with 3 QSSPCLK
	QSPI_QSSL_CLK4    = 0x0003,  //QSPI QSSL high level with 4 QSSPCLK
	QSPI_QSSL_CLK5    = 0x0004,  //QSPI QSSL high level with 5 QSSPCLK
	QSPI_QSSL_CLK6    = 0x0005,  //QSPI QSSL high level with 6 QSSPCLK
	QSPI_QSSL_CLK7    = 0x0006,  //QSPI QSSL high level with 7 QSSPCLK
	QSPI_QSSL_CLK8    = 0x0007,  //QSPI QSSL high level with 8 QSSPCLK
	QSPI_QSSL_CLK9    = 0x0008,  //QSPI QSSL high level with 9 QSSPCLK
	QSPI_QSSL_CLK10   = 0x0009,  //QSPI QSSL high level with 10 QSSPCLK
	QSPI_QSSL_CLK11   = 0x000A,  //QSPI QSSL high level with 11 QSSPCLK
	QSPI_QSSL_CLK12   = 0x000B,  //QSPI QSSL high level with 12 QSSPCLK
	QSPI_QSSL_CLK13   = 0x000C,  //QSPI QSSL high level with 13 QSSPCLK
	QSPI_QSSL_CLK14   = 0x000D,  //QSPI QSSL high level with 14 QSSPCLK
	QSPI_QSSL_CLK15   = 0x000E,	 //QSPI QSSL high level with 15 QSSPCLK
	QSPI_QSSL_CLK16   = 0x000F,  //QSPI QSSL high level with 16 QSSPCLK
}QSPI_SSLWidth_t;

typedef enum
{
	QSPI_SSLRel_half   		= 0,		//release QSSL after 0.5 QSPCLK
	QSPI_SSLRel_Onehalf   = 1,		//release QSSL after 1.5 QSPCLK
} QSPI_SSLRelease_t;

typedef enum
{
	QSPI_SSLOutInt_half   	 = 0,		//output QSSL before 0.5 QSPCLK
	QSPI_SSLOutInt_Onehalf   = 1,		//output QSSL before 1.5 QSPCLK
} QSPI_SSLOutInterval_t;

typedef enum
{
	QSPI_SCKPeriod_CLK2    = 0x0000,	 //SCK output period at 2 PCLKA
	QSPI_SCKPeriod_CLK3    = 0x0001,	 //SCK output period at 3 PCLKA
	QSPI_SCKPeriod_CLK4    = 0x0002,   //SCK output period at 4 PCLKA
	QSPI_SCKPeriod_CLK5    = 0x0003,   //SCK output period at 5 PCLKA
	QSPI_SCKPeriod_CLK6    = 0x0004,   //SCK output period at 6 PCLKA
	QSPI_SCKPeriod_CLK7    = 0x0005,   //SCK output period at 7 PCLKA
	QSPI_SCKPeriod_CLK8    = 0x0006,   //SCK output period at 8 PCLKA
	QSPI_SCKPeriod_CLK9    = 0x0007,   //SCK output period at 9 PCLKA
	QSPI_SCKPeriod_CLK10   = 0x0008,   //SCK output period at 10 PCLKA
	QSPI_SCKPeriod_CLK11   = 0x0009,   //SCK output period at 11 PCLKA
	QSPI_SCKPeriod_CLK12   = 0x000A,   //SCK output period at 12 PCLKA
	QSPI_SCKPeriod_CLK13   = 0x000B,   //SCK output period at 13 PCLKA
	QSPI_SCKPeriod_CLK14   = 0x000C,   //SCK output period at 14 PCLKA
	QSPI_SCKPeriod_CLK15   = 0x000D,   //SCK output period at 15 PCLKA
	QSPI_SCKPeriod_CLK16   = 0x000E,   //SCK output period at 16 PCLKA
	QSPI_SCKPeriod_CLK17   = 0x000F,   //SCK output period at 17 PCLKA
	QSPI_SCKPeriod_CLK18   = 0x0010,	 //SCK output period at 18 PCLKA
	QSPI_SCKPeriod_CLK20   = 0x0011,   //SCK output period at 20 PCLKA
	QSPI_SCKPeriod_CLK22   = 0x0012,   //SCK output period at 22 PCLKA
	QSPI_SCKPeriod_CLK24   = 0x0013,   //SCK output period at 24 PCLKA
	QSPI_SCKPeriod_CLK26   = 0x0014,   //SCK output period at 26 PCLKA
	QSPI_SCKPeriod_CLK28   = 0x0015,   //SCK output period at 28 PCLKA
	QSPI_SCKPeriod_CLK30   = 0x0016,   //SCK output period at 30 PCLKA
	QSPI_SCKPeriod_CLK32   = 0x0017,   //SCK output period at 32 PCLKA
	QSPI_SCKPeriod_CLK34   = 0x0018,	 //SCK output period at 34 PCLKA
	QSPI_SCKPeriod_CLK36   = 0x0019,   //SCK output period at 36 PCLKA	
	QSPI_SCKPeriod_CLK38   = 0x001A,   //SCK output period at 38 PCLKA
	QSPI_SCKPeriod_CLK40   = 0x001B,   //SCK output period at 40 PCLKA
	QSPI_SCKPeriod_CLK42   = 0x001C,   //SCK output period at 42 PCLKA
	QSPI_SCKPeriod_CLK44   = 0x001D,   //SCK output period at 44 PCLKA
	QSPI_SCKPeriod_CLK46   = 0x001E,   //SCK output period at 46 PCLKA
	QSPI_SCKPeriod_CLK48   = 0x001F,   //SCK output period at 48 PCLKA
} QSPI_SCKPeriod_t;

typedef enum
{
	QSPI_SCKCHK_None   = 0x0000,   //SCK duty do not correct
	QSPI_SCKCHK_Open   = 0x0001,   //SCK duty correct open 
} QSPI_SCKCorrect_t;

typedef enum
{
	QSPI_Mode_Rom   	 = 0x0000,   //ROM mode
	QSPI_Mode_Direct   = 0x0001,   //direct mode
} QSPI_AccessMode_t;

typedef enum
{
	QSPI_AddrWith_1byte   	 = 0x0000,   //QSPI address 1 byte
	QSPI_AddrWith_2byte   	 = 0x0001,   //QSPI address 2 byte
	QSPI_AddrWith_3byte   	 = 0x0002,   //QSPI address 3 byte
	QSPI_AddrWith_4byte   	 = 0x0003,   //QSPI address 4 byte
} QSPI_AddrWith_t;

typedef enum
{
	QSPI_ReadAddr_Normal   	 	 = 0x0000,   //do not use 4 bytes address read
	QSPI_ReadAddr_Addr4bytes   = 0x0001,   //use 4 bytes address read
} QSPI_ReadAddr4bc_t;

typedef enum
{
	QSPI_FalsePeriod_CLK0    = 0x0000,  //QSPI QSSL high level with default QSSPCLK
	QSPI_FalsePeriod_CLK3    = 0x0001,  //QSPI QSSL high level with 3 QSSPCLK
	QSPI_FalsePeriod_CLK4    = 0x0002,  //QSPI QSSL high level with 4 QSSPCLK
	QSPI_FalsePeriod_CLK5    = 0x0003,  //QSPI QSSL high level with 5 QSSPCLK
	QSPI_FalsePeriod_CLK6    = 0x0004,  //QSPI QSSL high level with 6 QSSPCLK
	QSPI_FalsePeriod_CLK7    = 0x0005,  //QSPI QSSL high level with 7 QSSPCLK
	QSPI_FalsePeriod_CLK8    = 0x0006,  //QSPI QSSL high level with 8 QSSPCLK
	QSPI_FalsePeriod_CLK9    = 0x0007,  //QSPI QSSL high level with 9 QSSPCLK
	QSPI_FalsePeriod_CLK10   = 0x0008,  //QSPI QSSL high level with 10 QSSPCLK
	QSPI_FalsePeriod_CLK11   = 0x0009,  //QSPI QSSL high level with 11 QSSPCLK
	QSPI_FalsePeriod_CLK12   = 0x000A,  //QSPI QSSL high level with 12 QSSPCLK
	QSPI_FalsePeriod_CLK13   = 0x000B,  //QSPI QSSL high level with 13 QSSPCLK
	QSPI_FalsePeriod_CLK14   = 0x000C,  //QSPI QSSL high level with 14 QSSPCLK
	QSPI_FalsePeriod_CLK15   = 0x000D,  //QSPI QSSL high level with 15 QSSPCLK
	QSPI_FalsePeriod_CLK16   = 0x000E,	//QSPI QSSL high level with 16 QSSPCLK
	QSPI_FalsePeriod_CLK17   = 0x000F,  //QSPI QSSL high level with 17 QSSPCLK
}QSPI_FalsePeriod_t;

typedef enum
{
	QSPI_XIP_Disable   	= 0x0000,   //diable XIP mode
	QSPI_XIP_Enable   	= 0x0001,   //enable XIP mode
} QSPI_XIP_t;

typedef enum
{
	QSPI_ProtocolType_Extend   	= 0x0000,   //extend SPI
	QSPI_ProtocolType_Dual    	= 0x0001,   //dual SPI
	QSPI_ProtocolType_Quad    	= 0x0002,   //quad SPI
} QSPI_ProtocolType_t;

typedef enum
{
	QSPI_WP_Low   		= 0x0000,   //WP low
	QSPI_WP_High    	= 0x0001,   //dual SPI
} QSPI_WP_t;

typedef enum
{
	QSPI_RMode_Standard   		= 0x0000,   //standard read
	QSPI_RMode_Quick		    	= 0x0001,   //quickly read
	QSPI_RMode_DualOut	    	= 0x0002,   //double line out when quickly read
	QSPI_RMode_DualIO		    	= 0x0003,   //double line io when quickly read
	QSPI_RMode_QuadOut	    	= 0x0004,   //four line out when quickly read
	QSPI_RMode_QuadIO		    	= 0x0005,   //four line io when quickly read
} QSPI_ReadMode_t;

typedef enum
{
	QSPI_SSLDelay_None    			= 0x0000,   //after SPI start, no extend QSSL
	QSPI_SSLDelay_Ext33		    	= 0x0001,   //after SPI start, extend QSSL to 33 QSPCLK
	QSPI_SSLDelay_Ext129	    	= 0x0002,   //after SPI start, extend QSSL to 129 QSPCLK
	QSPI_SSLDelay_Indefinite		= 0x0003,   //after SPI start, extend QSSL indefinitely
} QSPI_QSSLDelay_t;

typedef enum
{
	QSPI_SPIMode_SPI0    	= 0x0000,   		//SPI mode 0
	QSPI_SPIMode_SPI3		  = 0x0001,   		//SPI mode 3
} QSPI_SPIMode_t;

typedef struct
{
	QSPI_ProtocolType_t QSPIProtocolType;		 					 /*!< Specifies the QSPI protocol type. This parameter can be value of @ref QSPI_Type_t */
	
	QSPI_AccessMode_t QSPIAccMode;		 /*!< Specifies the QSPI mode. This parameter can be value of @ref QSPI_AccessMode_t */
	
	QSPI_XIP_t QSPIXIP;			   	       /*!< Specifies the QSPI XIP mode. This parameter can be value of @ref QSPI_XIP_t */
	
	QSPI_SPIMode_t QSPISPIMode;	       /*!< Specifies the QSPI SPI mode. This parameter can be value of @ref QSPI_SPIMode_t */
	
	QSPI_AddrWith_t QSPIAddrWidth;			 	 /*!< Specifies the QSPI address bytes. This parameter can be value of @ref QSPI_AddrWith_t */
	
	QSPI_ReadAddr4bc_t QSPIRAddr4BC;				 /*!< Specifies the QSPI read address . This parameter can be value of @ref QSPI_ReadAddr_t */
	
	QSPI_ReadMode_t	QSPIReadMode;        /*!< Specifies the QSPI read mode,This parameter can be a value of @ref QSPI_ReadMode_t.	*/
	
	QSPI_FalsePeriod_t QSPIFalsPeriod;        /*!< Specifies the QSPI false command period,This parameter can be a value of @ref QSPI_FalsePeriod_t.	*/
	
	QSPI_SSLWidth_t 	QSPISSLWidth;       	 /*!< Specifies the QSPI QSSL high level width. This parameter can be value of @ref QSPI_SSLWidth_t */
	
	QSPI_SSLRelease_t QSPIRelTim;       	 /*!< Specifies the QSPI QSSL release time. This parameter can be a value of @ref QSPI_SSLRelease_t */
	
  QSPI_SSLOutInterval_t QSPIOutTim;	   /*!< Specifies the QSPI output interval,This parameter can be a value of @ref QSPI_SSLOutInterval_t */

	QSPI_SCKPeriod_t QSPIPeriod;  		/*!< Specifies the QSPI SCK period,This parameter can be a value of @ref QSPI_SCKPeriod_t.	*/ 
									   	
	QSPI_SCKCorrect_t QSPIDutyCheck;     /*!< Specifies the QSPI SCK correct,This parameter can be a value of @ref QSPI_SCKCorrect_t.	*/ 

	QSPI_QSSLDelay_t QSPISSLExt;      /*!< Specifies the QSPI SSL extend set,This parameter can be a value of @ref QSPI_QSSLDelay_t.	*/ 
}QSPI_InitTypeDef; 

void SoftReset(void);
uint8_t ReadStatus(status_t name);
void WriteStatus(status_t name, uint8_t data);
void EnableQPI(void);
void DisableQPI(void);
void EraseSector(uint32_t adr);
void EraseBlock(uint32_t adr);
void EraseChip(void);
void ProgramPage(uint32_t adr, uint32_t sz, uint8_t *buf);

void QSPI_SwitchTime_Set(FunctionalState StatusSW);
void QSPI_WP_Set(QSPI_WP_t StatusWP);
void QSPI_Prefetch_Set(FunctionalState StatusP);
void QSPI_PrefetchStop_Position(FunctionalState StatusP);
void QSPI_CSExtend_Set(FunctionalState StatusE);
void QSPI_HighLevelWait_Set(FunctionalState StatusE);
void QSPI_LowLevelWait_Set(FunctionalState StatusE);
void QSPI_ReadMode_Set(QSPI_ReadMode_t RMod);
void QSPI_ReadCMDCode_Set(FunctionalState StatusE);
void QSPI_Protocol_Set(QSPI_ProtocolType_t Type);
void QSPI_QSSLDelay_Set(QSPI_QSSLDelay_t Type);
void QSPI_XIP_Set(FunctionalState Status);
void qspi_Init(QSPI_InitTypeDef* QSPI_InitStruct);
#endif
