/**
  ******************************************************************************
  * @file    qspi.c
  * @author  CMS Application Team
  * @version Vx.x.x
  * @date    24-April-2022
  * @brief   This file provides firmware functions to manage the following 
  *          functionalities of the Quad Serial Perripheral Interface(qspi):           
  @verbatim       
 ===============================================================================
                        ##### How to use this driver #####
 ===============================================================================
    [..]
            
    @endverbatim        
  ******************************************************************************
  * @attention
  *
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "qspi.h"


/** @addtogroup cms32h3201_StdPeriph_Driver
  * @{
  */

/** @defgroup QSPI 
  * @brief QSPI driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  soft reset norflash.
  * @param  None
  * @retval None
  */
void SoftReset(void)
{
	QSPI->SFMSMD= 0x01U; // Direct communication mode 
	QSPI->SFMCOM = 0x66U; // Enable Reset
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCOM = 0x99U; // Reset
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
	
}

/**
  * @brief  read norflash register1 and register2.
  * @param  name:STATUS_REG1,STATUS_REG2
  * @retval None
  */
uint8_t ReadStatus(status_t name)
{
	uint8_t tmp;
	
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	if(name == STATUS_REG1)
		QSPI->SFMCOM = 0x05U; // Read Status Register-1
	else
		QSPI->SFMCOM = 0x35U; // Read Status Register-2
	tmp = (uint8_t)QSPI->SFMCOM; // Read Status
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
	return tmp;
}

/**
  * @brief  write norflash register1 and register2.
  * @param  name:STATUS_REG1,STATUS_REG2
  * @retval None
  */
void WriteStatus(status_t name, uint8_t data)
{
	
	QSPI->SFMSSC = 0x0FU; // the leading/trailing edge of SSB is 0.5 SCK, SSB high-level width 16 SCK
	QSPI->SFMCMD = 0x01U; // Direct communication mode 
	QSPI->SFMCOM = 0x06U; // Write Enable Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	if(name == STATUS_REG1)
		QSPI->SFMCOM = 0x01U; // Write Status Register-1
	else
		QSPI->SFMCOM = 0x31U; // Write Status Register-2
	
	QSPI->SFMCOM = data;  // Write Status
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x05U; // Read Status Register-1 Instruction
	while(QSPI->SFMCOM & BUSY_Msk);
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  enable norflash QPI.
	* @param  None
  * @retval None
  */
void EnableQPI(void)
{
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0x38U; // Enable QPI
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  disable norflash QPI.
	* @param  None
  * @retval None
  */
void DisableQPI(void)
{
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0xFFU; // Disable QPI
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  erase sector of norflash by QSPI.
	* @param  None
  * @retval None
  */
void EraseSector(uint32_t adr)
{
	QSPI->SFMSSC = 0x0FU; // the leading/trailing edge of SSB is 0.5 SCK, SSB high-level width 16 SCK
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0x06U; // Write Enable Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x20U; // Sector Erase Instruction
	QSPI->SFMCOM = BYTE_FIELD(adr, 2); // Address [23:16] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 1); // Address [15:08] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 0); // Address [07:00] 
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x05U; // Read Status Register-1 Instruction
	while(QSPI->SFMCOM & BUSY_Msk);
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  erase block of norflash by QSPI.
* @param  adr:the start address of block.
  * @retval None
  */
void EraseBlock(uint32_t adr)
{
	QSPI->SFMSSC = 0x0FU; // the leading/trailing edge of SSB is 0.5 SCK, SSB high-level width 16 SCK
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0x06U; // Write Enable Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x52U; // 16KB Block Erase Instruction
	QSPI->SFMCOM = BYTE_FIELD(adr, 2); // Address [23:16] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 1); // Address [15:08] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 0); // Address [07:00] 
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	// polling BUSY Flag 
	QSPI->SFMCOM = 0x05U; // Read Status Register-1 Instruction
	while(QSPI->SFMCOM & BUSY_Msk);
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  erase chip of norflash by QSPI.
  * @param  None.
  * @retval None
  */
void EraseChip(void)
{
	uint8_t tmp;

	QSPI->SFMSSC = 0x0FU; // the leading/trailing edge of SSB is 0.5 SCK, SSB high-level width 16 SCK
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0x06U; // Write Enable Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x60U; // Chip Erase Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	// polling BUSY Flag 
	do {
	    QSPI->SFMCOM = 0x05U; // Read Status Register-1 Instruction
		tmp = (uint8_t)QSPI->SFMCOM;
	    QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	} while(tmp & BUSY_Msk);
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  write norflash by QSPI.
  * @param  adr: norflash address.
  * @param  sz: write size.
  * @param  buf: data.
  * @retval None
  */
void ProgramPage(uint32_t adr, uint32_t sz, uint8_t *buf)
{
	QSPI->SFMSSC = 0x0FU; // the leading/trailing edge of SSB is 0.5 SCK, SSB high-level width 16 SCK
	QSPI->SFMCMD = 0x01U; // Direct communication mode
	QSPI->SFMCOM = 0x06U; // Write Enable Instruction
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	QSPI->SFMCOM = 0x02U; // Page Program Instruction
	QSPI->SFMCOM = BYTE_FIELD(adr, 2); // Address [23:16] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 1); // Address [15:08] 
	QSPI->SFMCOM = BYTE_FIELD(adr, 0); // Address [07:00] 

	while(sz--)
	{
		QSPI->SFMCOM = *buf++; // Write Data Byte
	}
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle

	// polling BUSY Flag 
	QSPI->SFMCOM = 0x05U; // Read Status Register-1 Instruction
	while(QSPI->SFMCOM & BUSY_Msk);
	QSPI->SFMCMD = 0x01U; // close SPI bus cycle
	QSPI->SFMCMD = 0x00U; // ROM access mode
}

/**
  * @brief  set the QSPI min switch time.
  * @param  StatusSW: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_SwitchTime_Set(FunctionalState StatusSW)
{
	if(StatusSW == ENABLE)
		QSPI->SFMSPC |= QSPI_SFMSPC_SFMSDE_Msk;
	else
		QSPI->SFMSPC &= ~QSPI_SFMSPC_SFMSDE_Msk;
}

/**
  * @brief  set the QSPI WP.
  * @param  StatusWP: QSPI_WP_High,QSPI_WP_Low.
  * @retval None
  */
void QSPI_WP_Set(QSPI_WP_t StatusWP)
{
	if(StatusWP == QSPI_WP_High)
		QSPI->SFMPMD |= QSPI_SFMPMD_SFMWPL_Msk;
	else
		QSPI->SFMPMD &= ~QSPI_SFMPMD_SFMWPL_Msk;
}

/**
  * @brief  set the QSPI prefetch time.
  * @param  StatusSW: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_Prefetch_Set(FunctionalState StatusP)
{
	if(StatusP == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMPFE_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMPFE_Msk;
}

/**
  * @brief  set the QSPI prefetch stop position set.
  * @param  StatusP: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_PrefetchStop_Position(FunctionalState StatusP)
{
	if(StatusP == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMPAE_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMPAE_Msk;
}

/**
  * @brief  set the QSPI enable signal extend.
  * @param  StatusE: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_CSExtend_Set(FunctionalState StatusE)
{
	if(StatusE == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMOEX_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMOEX_Msk;
}

/**
  * @brief  set the QSPI high level wait when communication.
  * @param  StatusE: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_HighLevelWait_Set(FunctionalState StatusE)
{
	if(StatusE == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMOHW_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMOHW_Msk;
}

/**
  * @brief  set the QSPI low level wait when communication.
  * @param  StatusE: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_LowLevelWait_Set(FunctionalState StatusE)
{
	if(StatusE == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMOSW_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMOSW_Msk;
}

/**
  * @brief  set the QSPI read mode.
  * @param  RMod: QSPI_RMode_Standard,QSPI_RMode_Quick,QSPI_RMode_DualOut,
  *               QSPI_RMode_DualIO,QSPI_RMode_QuadOut,QSPI_RMode_QuadIO.
  * @retval None
  */
void QSPI_ReadMode_Set(QSPI_ReadMode_t RMod)
{
	uint32_t temp=0;
	
	temp = QSPI->SFMSMD;
	temp &= ~(QSPI_SFMSMD_SFMRM_Msk);
	temp |= RMod;
	QSPI->SFMSMD = temp;
}

/**
  * @brief  set the QSPI read command code or not.
  * @param  StatusE: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_ReadCMDCode_Set(FunctionalState StatusE)
{
	if(StatusE == ENABLE)
		QSPI->SFMSMD |= QSPI_SFMSMD_SFMCCE_Msk;
	else
		QSPI->SFMSMD &= ~QSPI_SFMSMD_SFMCCE_Msk;
}

/**
  * @brief  set the QSPI work protocol type.
  * @param  Type: QSPI_ProtocolType_Extend,QSPI_ProtocolType_Dual,QSPI_ProtocolType_Quad.
  * @retval None
  */
void QSPI_Protocol_Set(QSPI_ProtocolType_t Type)
{
	uint32_t temp=0;
	
	temp = QSPI->SFMSPC;
	temp &= ~(QSPI_SFMSPC_SFMSPI_Msk);
	temp |= Type;
	QSPI->SFMSPC = temp;
}

/**
  * @brief  set the QSPI QSSL delay.
  * @param  Type: QSPI_SSLDelay_None,QSPI_SSLDelay_Ext33,QSPI_SSLDelay_Ext129,QSPI_SSLDelay_Indefinite.
  * @retval None
  */
void QSPI_QSSLDelay_Set(QSPI_QSSLDelay_t Type)
{
	uint32_t temp=0;
	
	temp = QSPI->SFMSMD;
	temp &= ~(QSPI_SFMSMD_SFMSE_Msk);
	temp |= Type << QSPI_SFMSMD_SFMSE_Pos;
	QSPI->SFMSMD = temp;
}

/**
  * @brief  set the QSPI XIP enable or not.
  * @param  Status: ENABLE,DISABLE.
  * @retval None
  */
void QSPI_XIP_Set(FunctionalState Status)
{
	
	if(Status == ENABLE)
	{
		QSPI->SFMSDC &= 0xAFFF;
		QSPI->SFMSDC |= QSPI_SFMSDC_SFMXEN_Msk;		
	}
	else
	{
		QSPI->SFMSDC |=0xFF00;
		QSPI->SFMSDC &= ~QSPI_SFMSDC_SFMXEN_Msk;
	}
}

/**
  * @brief  initial QSPI.
  * @param  QSPI_InitStruct.
  * @retval None
  */
void qspi_Init(QSPI_InitTypeDef* QSPI_InitStruct)
{
	uint32_t temp=0;
	
	/**********set QSPI read mode, QSSL extend clock and SPI mode**********************************/	
	temp = QSPI->SFMSMD;
	temp &= ~(QSPI_SFMSMD_SFMRM_Msk | QSPI_SFMSMD_SFMSE_Msk | QSPI_SFMSMD_SFMMD3_Msk);
	temp |= QSPI_InitStruct->QSPIReadMode | (QSPI_InitStruct->QSPISSLExt << QSPI_SFMSMD_SFMSE_Pos)
        	| (QSPI_InitStruct->QSPISPIMode << QSPI_SFMSMD_SFMMD3_Pos);
	QSPI->SFMSMD = temp;

/**********set QSPI QSSL width, release time and output time**********************************/		
	temp = QSPI->SFMSSC;
	temp &= ~(QSPI_SFMSSC_SFMSW_Msk | QSPI_SFMSSC_SFMSHD_Msk | QSPI_SFMSSC_SFMSLD_Msk);
	temp |= QSPI_InitStruct->QSPISSLWidth | (QSPI_InitStruct->QSPIRelTim << QSPI_SFMSSC_SFMSHD_Pos)
        	| (QSPI_InitStruct->QSPIOutTim << QSPI_SFMSSC_SFMSLD_Pos);
	QSPI->SFMSSC = temp;

/**********set QSPI period and duty correct***************************************************/		
	temp = QSPI->SFMSKC;
	temp &= ~(QSPI_SFMSKC_SFMDV_Msk | QSPI_SFMSKC_SFMDTY_Msk);
	temp |= QSPI_InitStruct->QSPIPeriod | (QSPI_InitStruct->QSPIDutyCheck << QSPI_SFMSKC_SFMDTY_Pos);
	QSPI->SFMSKC = temp;

/**********set QSPI access mode***************************************************/		
	if(QSPI_InitStruct->QSPIAccMode)
		QSPI->SFMCMD |= QSPI_SFMCMD_DCOM_Msk;		//direct mode
	else
		QSPI->SFMCMD &= ~QSPI_SFMCMD_DCOM_Msk;	//ROM mode
	
/**********set QSPI address width*************************************************/			
	temp = QSPI->SFMSAC;
	temp &= ~(QSPI_SFMSAC_SFMAS_Msk | QSPI_SFMSAC_SFM4BC_Msk);
	temp |= QSPI_InitStruct->QSPIAddrWidth | (QSPI_InitStruct->QSPIRAddr4BC << QSPI_SFMSAC_SFM4BC_Pos);
	QSPI->SFMSAC = temp;
	
/**********set the XIP and false command period ***********************************/		
	temp = QSPI->SFMSDC;
	temp &= ~(QSPI_SFMSDC_SFMDN_Msk | QSPI_SFMSDC_SFMXEN_Msk);
	temp |= QSPI_InitStruct->QSPIFalsPeriod | (QSPI_InitStruct->QSPIXIP << QSPI_SFMSDC_SFMXEN_Pos);
	QSPI->SFMSDC = temp;
	
/**********set the QSPI work mode *************************************************/	
	temp = QSPI->SFMSPC;
	temp &= ~(QSPI_SFMSPC_SFMSPI_Msk);
	temp |= QSPI_InitStruct->QSPIProtocolType;
	QSPI->SFMSPC = temp;
	
}



